(*pp ./tracepp *)

let perms = 0o644

let fix_database_filename fname = 
  if not (StrExtras.suffix ".db" fname) then
    raise (Dbm.Dbm_error ("Database file has not the suffix .db: " ^ fname))
  else
    String.sub fname 0 (String.length fname - 3)


let with_write fname f = 
  let fixed_fname = fix_database_filename fname in 
  let db = Dbm.opendbm fixed_fname [Dbm.Dbm_rdwr; Dbm.Dbm_create] perms in
    try (let res = f db in Dbm.close db; res)
    with e -> (Dbm.close db; raise e)
  
let with_read fname f = 
  let fixed_fname = fix_database_filename fname in 
  let db =  Dbm.opendbm fixed_fname [Dbm.Dbm_rdonly] perms in
    try (let res = f db in Dbm.close db; res)
    with e -> (Dbm.close db; raise e)

(* public
   val find : t -> string -> string *)
let find = Dbm.find

(* public
   val search : t -> string -> string -> string *)
let search db key default = 
  try Dbm.find db key
  with Not_found -> default

(* public
   val exists : t -> string -> bool *)
let exists db key = 
  try let _ = Dbm.find db key in true
  with Not_found              -> false

(* public
   val add : t -> string -> string -> unit *)
let add = Dbm.add

(* public 
   val replace : t -> string -> string -> unit *)
let replace = Dbm.replace

(* public
   val remove : t -> string -> unit *)
let remove = Dbm.remove

(* public
   val firstkey : t -> string *)
let firstkey = Dbm.firstkey

(* public
   val nextkey : t -> string *)
let nextkey = Dbm.nextkey

(* public
   val iter : (string -> string -> 'a) -> t -> unit *)
let iter = Dbm.iter

(* public
   (* similar to iter, but the results of the function are not discarded
      but returned in a list *)
   val map : (string -> string -> 'a) -> t -> 'a list *)
let map f t =
  let rec walk acc next =
    match next with
        None -> acc
      | Some k ->
          let res = f k (find t k) in
            walk (res :: acc) (try Some(nextkey t) with Not_found -> None)
  in
    List.rev (walk [] (try Some(firstkey t) with Not_found -> None))

(* public
   val fold_left ('a -> string -> string -> 'b) -> 'a -> t -> 'b *)
let fold_left f init t = 
  let rec walk res next = 
    match next with
        None -> res
      | Some k ->
          let res' = f res k (find t k) in
            walk res' (try Some(nextkey t) with Not_found -> None)
  in
    walk init (try Some(firstkey t) with Not_found -> None)
