{ (*pp ./tracepp *)
  open Parser
  open Lexing

  exception Internal_lexer_error of string
  
  let get_filename lexbuf = 
    let start_pos = lexeme_start_p lexbuf in
      start_pos.pos_fname

  let get_lineno lexbuf = 
    let start_pos = lexeme_start_p lexbuf in
      start_pos.pos_lnum

  let adjust_lineno lexbuf = 
    let start_pos = lexeme_start_p lexbuf in
    let old = start_pos.pos_lnum in
    let n = Utils.count_char (lexeme lexbuf) '\n' in
      lexbuf.lex_curr_p <- { lexbuf.lex_curr_p with pos_lnum = old + n }
      
}

let word = [^ ' ' '\t' '\n']+
let tab_or_space = [' ' '\t']
let maybe_voc_sep = (tab_or_space* '\n')+ 
  
rule voc_sep_lexer = parse
    tab_or_space+   { BLANK }
  | eof             { EOF }
  | _               { VOC_START (lexeme_char lexbuf 0, 
                                 (get_filename lexbuf,
                                  get_lineno lexbuf)) }

and lexer = parse
    "->"            { RIGHT_ARROW }
  | "<-"            { LEFT_ARROW }
  | "<->"           { LEFT_RIGHT_ARROW }
  | '{'             { LBRACE }
  | '}'             { RBRACE }
  | "Ex:"           { EXAMPLE }
  | maybe_voc_sep   { adjust_lineno lexbuf; voc_sep_lexer lexbuf }
  | tab_or_space+   { BLANK }
  | eof             { EOF }
  | _               { CHAR (lexeme_char lexbuf 0) } 
            
{

  (* make_lexer creates a lexer which manipulates the token stream
     by inserting a VOC_START token at the beginning (if not 
     already present) and by deleting certain BLANKs.

     This may seem a bit ugly, because the lexer has something 
     to do with the grammer, but is has shown that this is the 
     easiest way. *)


  let next_non_blank lexbuf = 
    let rec forward () = 
      match lexer lexbuf with
          BLANK -> forward ()
        | x     -> x
    in forward ()

  let make_lexer () = 
    let start_flag = ref true in
    let f lexbuf =
      let next_tok = (
        if not !start_flag then
          match lexer lexbuf with
              BLANK -> (
                match next_non_blank lexbuf with
                    CHAR c -> WORD_START c
                  | x      -> x
              )
            | x -> x
        else
          (start_flag := false;
           match lexer lexbuf with
               CHAR c      -> VOC_START (c, (get_filename lexbuf, 
                                             get_lineno lexbuf))
             | VOC_START x -> VOC_START x
             | x           -> 
                 TRACE Trace.lex "Error, token stream starts with %s" 
                   (Token.to_string x);
                 raise Parsing.Parse_error)
      ) in (TRACE Trace.lex "%s" (Token.to_string next_tok); next_tok)
    in (TRACE Trace.lex "start_flag = %b" !start_flag; f)
            
}
