%{
  (* header *)
  
  open Vocable

  type side = {text: Vocable.words; pronunciation: string option}
  type direction = Left | Right | Both

  let make_voc (fname, lineno) left dir right exs = 
    let make_voc' term meaning =
      {voc_term = term.text; 
       voc_termPronunciation = term.pronunciation;
       voc_meaning = meaning.text; 
       voc_meaningPronunciation = meaning.pronunciation;
       voc_examples = exs;
       voc_filename = Filename.basename fname;
       voc_lineno = lineno;
       voc_index = None;
       voc_time = None;
       voc_timesAsked = 0}
    in 
      if dir == Right then
        [make_voc' left right]
      else if dir == Left then
        [make_voc' right left]
      else
        [make_voc' left right; make_voc' right left]
    

  let prepend_char c s = 
    let n = String.length s in
    let buf = String.create (n + 1) in
      String.blit s 0 buf 1 n;
      buf.[0] <- c;
      buf

  let prepend_char_to_side c s =
    match s.text with
        []      -> {s with text = [String.make 1 c]}
      | w :: ws -> {s with text = prepend_char c w :: ws}

%}
/* declarations */

%token EOF BLANK RIGHT_ARROW LEFT_ARROW LEFT_RIGHT_ARROW LBRACE RBRACE EXAMPLE 

%token <char> CHAR WORD_START
%token <char * (string * int)> VOC_START

%start main
%type <Vocable.t list> main

%%
/* rules */

main : vocables EOF  { $1 }

vocables :
  { [] }
| vocable vocables  { List.append $1 $2 }

vocable : 
  left_side direction right_side examples { let left, pos = $1 in
                                              make_voc pos left $2 $3 $4 }

examples :  
  { [] } 
| example examples { $1 :: $2 }

example : EXAMPLE words { $2 }

words :
  word          { [$1] }
| word words   { $1 :: $2 }

word : 
  CHAR word2        { StrExtras.implode ($1 :: $2) }
| WORD_START word2  { StrExtras.implode ($1 :: $2) }

word2 :
  { [] }
| CHAR word2  { $1 :: $2 }

right_side : side { $1 }

left_side : 
  VOC_START      { let c, pos = $1 in
                     ({text = [String.make 1 c]; pronunciation = None},
                      pos) }
| VOC_START side { let c, pos = $1 in
                     (prepend_char_to_side c $2, pos) }

side : 
  words pronunciation { {text = $1; pronunciation = Some $2} }
| words               { {text = $1; pronunciation = None} }

direction :
  LEFT_ARROW        { Left }
| RIGHT_ARROW       { Right }
| LEFT_RIGHT_ARROW  { Both }

pronunciation : LBRACE word RBRACE  { $2 }

%%
(* trailer *)
