(*pp ./tracepp *)

type session = { ses_session_file: string;
                 ses_db_file: string;
                 ses_vocable_files: string list }

let session_to_string ses = 
  "{ ses_session_file: " ^ ses.ses_session_file ^ "\n" ^
  "  ses_db_file: " ^ ses.ses_db_file ^ "\n" ^
  "  ses_vocable_files: " ^ 
  ToString.from_string_list ses.ses_vocable_files ^ " }"

(* Gets the real filename of the file 'voc' is contained in.
   Only the simle filename is stored in the vocable record,
   so this function resolves the simple name against
   the directory of the session file. *)
let get_voc_filename ses voc = 
  let dir = Filename.dirname (ses.ses_session_file) in
  let fname = voc.Vocable.voc_filename in
    Filename.concat dir fname

let get_audio_filename ses simple_audio_name =
  let dir = Filename.dirname (ses.ses_session_file) in
    Filename.concat (Filename.concat dir "audio") simple_audio_name

(* Constructs a session record from the information contained
   in the session file.
   Takes care that the vocable files are interpreted relatively
   to the session file (not the cwd). *)
let deserialize_session session_file =
  let _ = ITRACE "Deserializing session from %s" session_file in
  let adjust_fname fname = 
    let dir = Filename.dirname fname in
      if dir <> "" && dir <> "." then (
        FATAL 
"Session file can contain only simple filenames. '%s' violates
this rule." fname;
        ""
      ) else
        Filename.concat (Filename.dirname session_file) fname
  in
  let chan = open_in session_file in
  let rec read_lines acc = 
    try
      let line = Utils.strip (input_line chan) in
        read_lines (adjust_fname line :: acc)
    with End_of_file -> acc
  in { ses_session_file = session_file;
       ses_db_file = session_file ^ ".db";
       ses_vocable_files = read_lines [] }
