let answer_stats = ref (0, 0)

let register_answer correct = 
  let correct_old, wrong_old = !answer_stats in
    answer_stats := (if correct then
                       correct_old + 1, wrong_old
                     else
                       correct_old, wrong_old + 1)


let display_answer_stats () =
  let correct, wrong = !answer_stats in
    Printf.printf "Questions answered correct: %d
Questions answered wrong: %d\n" correct wrong

let display_voc_stats voc = 
  let i = 
    match voc.Vocable.voc_index with
        Some i -> string_of_int i
      | None   -> "(never asked)"
  in
  let t = 
    match voc.Vocable.voc_time with
        Some t -> Utils.string_of_tstamp t
      | None   -> "(never asked)"
  in
  let fname = voc.Vocable.voc_filename in
  let lineno = string_of_int voc.Vocable.voc_lineno in
    print_endline ("Filename:        " ^ fname);
    print_endline ("Line number:     " ^ lineno);
    print_endline ("Index:           " ^ i);
    print_endline ("Last time asked: " ^ t)


let display_global_stats session =
  let db_file = session.Session.ses_db_file in
  let index_arr = Array.make 8 0 in
  let extreme_times = ref None in
  let total = ref 0 in
  let f key value = 
    let voc = Vocable.deserialize value in
    let i =
      match voc.Vocable.voc_index with
          None -> 0
        | Some i -> min i ((Array.length index_arr) - 1)
    in
      (match voc.Vocable.voc_time with
           None -> ()
         | Some t -> 
             match !extreme_times with
                 Some (least_recent, most_recent) ->
                   extreme_times := Some (min t least_recent, 
                                          max t most_recent)
               | None ->
                   extreme_times := Some (t, t)
      );
      index_arr.(i) <- index_arr.(i) + 1;
      total := !total + 1
  in 
    Database.with_read db_file (fun db -> Database.iter f db);
    let least_recent, most_recent = 
      match !extreme_times with
          Some (x, y) -> (Utils.string_of_tstamp x, Utils.string_of_tstamp y)
        | None        -> ("<Unknown>", "<Unknown>")
    in
    let display_indices i times = 
      let percentage = (float_of_int times /. float_of_int !total) *. 100.0 in
      let fmt = format_of_string "%3.1f%% (%d)\n" in
        if i == 0 then
          Printf.printf ("Number of vocable never asked: " ^^ fmt)
            percentage times
        else if i <> (Array.length index_arr) - 1 then
          Printf.printf ("Number of vocables with index %d: " ^^ fmt)
            i percentage times
        else
          Printf.printf ("Number of vocables with index equal or higher " ^^
                         "than %d: " ^^ fmt) i percentage times
    in
      Printf.printf "Total number of vocables: %d
'Youngest' vocable asked on %s
'Oldest' vocable asked on %s
" !total most_recent least_recent;
      Array.iteri display_indices index_arr

