(*pp ./tracepp *)

(* 'create_hashtbl n' creates a hash table for n items *)
let create_hashtbl n = 
  let n = float n in
  let size = int_of_float (n ** 1.41) in
    Hashtbl.create size

(* computes the number of occurences of c in s *)
let count_char s c = 
  StrExtras.fold_left (fun n c2 -> if c = c2 then n + 1 else n) 0 s

let rec float_eq x y = 
  if x > y then float_eq y x
  else x +. epsilon_float >= y && y -. epsilon_float <= x

let string_of_tstamp t =
  Time.format_time "%Y-%m-%d %H:%M:%S" t

let string_of_tdiff secs = 
  let div_mod x y = 
    (Int32.div x y, Int32.rem x y) 
  in
  let str x dscr = 
    if x = Int32.one then
      Int32.to_string x ^ " " ^ dscr
    else
      Int32.to_string x ^ " " ^ dscr ^ "s"
  in
  let h, hmod = div_mod secs (Int32.of_int (60 * 60)) in
  let m, s = div_mod hmod (Int32.of_int 60) in
    if h = Int32.zero then
      if m = Int32.zero then
        str s "second"
      else
        str m "minute" ^ ", " ^ str s "second"
    else
      str h "hour" ^ ", " ^ str m "minute" ^ ", " ^ str s "second"

let strip s = 
  let n = String.length s in
  let rec f i =
    if i >= n then ""
    else if CharExtras.is_space s.[i] then f (i + 1)
    else StrExtras.chomp (StrExtras.cut_first_n s i)
  in f 0

