(*pp ./tracepp *)

open ToString

type words = string list


(* !!! DON'T FORGET TO MAKE A NEW VERSION IF YOU EDIT SOMETHING HERE !!! *)
type t = { voc_term: words;
           voc_meaning: words;
           voc_examples: words list;
           voc_termPronunciation: string option;
           voc_meaningPronunciation: string option;
           voc_filename: string;
           voc_lineno: int;
           voc_index: int option;
           voc_time: Int32.t option;
           voc_timesAsked: int }
(* !!! DON'T FORGET TO MAKE A NEW VERSION IF YOU EDIT SOMETHING HERE !!! *)


let get_term voc = from_string_list voc.voc_term
let get_meaning voc = from_string_list voc.voc_meaning
let get_examples voc = List.map from_string_list voc.voc_examples

let update_index_and_time voc i t = 
  { voc with voc_index = i; voc_time = t}

let term_str voc = 
  "term                 = " ^ get_term voc
    
let meaning_str voc = 
  "meaning              = " ^ get_meaning voc

let examples_str voc = 
  "examples             = " ^ 
   from_list ~sep:"; " from_string_list voc.voc_examples 

let termPronunciation_str voc =
  "termPronunciation    = " ^ 
  from_string_option ~none:"--" voc.voc_termPronunciation

let meaningPronunciation_str voc = 
  "meaningPronunciation = " ^ 
  from_string_option ~none:"--" voc.voc_meaningPronunciation

let filename_str voc =
  "filename             = " ^ voc.voc_filename

let lineno_str voc =
  "lineno               = " ^ string_of_int voc.voc_lineno

let index_str voc =
  "index = " ^ 
  from_option string_of_int ~none:"--" voc.voc_index

let time_str voc = 
  "time = " ^
  from_option Utils.string_of_tstamp ~none:"--" voc.voc_time

let times_asked_str voc = 
  "timesAsked = " ^ string_of_int voc.voc_timesAsked

let voc_to_string voc = 
  ("{ " ^ term_str voc ^ ";\n" ^
   "  " ^ meaning_str voc ^ ";\n" ^
   "  " ^ examples_str voc ^ ";\n" ^
   "  " ^ termPronunciation_str voc ^ ";\n" ^
   "  " ^ meaningPronunciation_str voc ^ ";\n" ^
   "  " ^ filename_str voc ^ ";\n" ^
   "  " ^ lineno_str voc ^ ";\n" ^
   "  " ^ index_str voc ^ ";\n" ^
   "  " ^ time_str voc ^ ";\n" ^
   "  " ^ times_asked_str voc ^ " }")

(*
voc_term: words;
           voc_meaning: words;
           voc_examples: words list;
           voc_termPronunciation: string option;
           voc_meaningPronunciation: string option;
           voc_filename: string;
           voc_lineno: int;
           voc_index: int option;
           voc_time: Int32.t option
  let buf1 = Buffer.create 8 in
  let _ = buf1.add_string "{ " in
  let buf2 = Buffer.create 8 in
  let _ = buf2.add_string "{ " in
*)
let diff voc1 voc2 = 
  let l1 = ref [] in
  let l2 = ref [] in
  let add f = 
    l1 := f voc1 :: !l1;
    l2 := f voc2 :: !l2
  in
    if voc1.voc_term <> voc2.voc_term then 
      add term_str
    ;
    if voc1.voc_meaning <> voc2.voc_meaning then 
      add meaning_str
    ;
    if voc1.voc_examples <> voc2.voc_examples then 
      add examples_str
    ;
    if voc1.voc_termPronunciation <> voc2.voc_termPronunciation then 
      add termPronunciation_str
    ;
    if voc1.voc_meaningPronunciation <> voc2.voc_meaningPronunciation then 
      add meaningPronunciation_str
    ;
    if voc1.voc_filename <> voc2.voc_filename then 
      add filename_str
    ;
    if voc1.voc_lineno <> voc2.voc_lineno then 
      add lineno_str
    ;
    if voc1.voc_index <> voc2.voc_index then 
      add index_str
    ;
    if voc1.voc_time <> voc2.voc_time then 
      add time_str
    ;
    if voc1.voc_timesAsked <> voc2.voc_timesAsked then
      add times_asked_str
    ;
    match (!l1, !l2) with
        ([], []) -> None
      | (l1, l2) -> 
          let res1 = "{ " ^ 
                     ToString.from_string_list ~sep:";\n" (List.rev l1) ^
                     "}" in
          let res2 = "{ " ^ 
                     ToString.from_string_list ~sep:";\n" (List.rev l2) ^
                     "}" in
            Some (res1, res2)
                       
                       
    
type vocable_1 = { voc_1_term: words;
                   voc_1_meaning: words;
                   voc_1_examples: words list;
                   voc_1_termPronunciation: string option;
                   voc_1_meaningPronunciation: string option;
                   voc_1_filename: string;
                   voc_1_lineno: int;
                   voc_1_index: int option;
                   voc_1_time: Int32.t option }
type vocable_2 = t

let lowest_version = 1
let current_version = 2

let magic = "ed39x7xq"

exception Deserialization_error of string

(*
  Description of the binary format for vocables:

  The binary representation of a vocable looks like this:

  HEADER MARSHALLED_DATA

  HEADER:
    magic number (8 letters)
    '_'
    version number (variable number of digits)
    '$'

  MARSHALLED_DATA:
    This is the result of applying Marshal.to_string to the
    record matching the version from the header.
  
  For deserialization, the version number decides to what
  version of the vocable record the data is unmarshalled. After
  unmarshalling, the resulting record is converted to the
  most current version of the record.
*)

(* val serialize : t -> string *)
let serialize voc = 
  let _ = DTRACE "Serializing vocable"  in
  let data = Marshal.to_string voc [] in
  let header = magic ^ "_" ^ string_of_int current_version ^ "$" in
    header ^ data
  
(* val deserialize_1 : string -> t *)
let deserialize_1 s =
  let _ = DTRACE "Deserializing vocable from version 1" in
  let v1 = (Marshal.from_string s 0 : vocable_1) in
    ({ voc_term                  = v1.voc_1_term;
       voc_meaning               = v1.voc_1_meaning;
       voc_examples              = v1.voc_1_examples;
       voc_termPronunciation     = v1.voc_1_termPronunciation;
       voc_meaningPronunciation  = v1.voc_1_meaningPronunciation;
       voc_filename              = v1.voc_1_filename;
       voc_lineno                = v1.voc_1_lineno;
       voc_index                 = v1.voc_1_index;
       voc_time                  = v1.voc_1_time;
       voc_timesAsked            = match v1.voc_1_index with
                                     Some i -> i
                                   | None -> 0
     } : t)
  
let deserialize_2 s =
  let _ = DTRACE "Deserializing vocable from version 2" in
  let v2 = (Marshal.from_string s 0 : vocable_2) in
    (v2 : t)

(* val deserialize : string -> t *)
let deserialize s = 
  let _ = DTRACE "Deserializing vocable" in
  let pref = (magic ^ "_") in
    if not (StrExtras.prefix pref s) then
      raise (Deserialization_error "string does not start with magic number")
    else (
      let vstart = String.length pref in
      let dollar = 
        try String.index_from s vstart '$' with
            Not_found -> 
              raise (Deserialization_error 
                       "version number not terminated by $")
      in let vstr = String.sub s vstart (dollar - vstart) in
      let version = 
        try int_of_string vstr with
            Failure _ ->
              raise (Deserialization_error "version number not an int")
      in let data =
          let start = dollar + 1 in
          let len = (String.length s) - start in
            String.sub s start len
      in 
      let voc = 
        ((match version with
              1 -> deserialize_1 data
            | 2 -> deserialize_2 data
            | _ -> raise (Deserialization_error ("illegal version: " ^ 
                                                 string_of_int version))
         ) : t)
      in
        DTRACE "Deserialization done";
        voc
    )


