(*pp ./tracepp *)

open Vocable

let square x = x * x

(* I cannot believe it, but I couldn'f find a function for
   computing a^n where a, n are integers *)
let rec power a n = 
  if n < 0 then invalid_arg ("power " ^ string_of_int a ^ " " ^
                             string_of_int n)
  else if n == 0 then 1
  else 
    let x = power a (n / 2) in
      if n mod 2 == 0 then x * x
      else a * x * x
 
let minutes n = 60 * n
let hours n = 60 * 60 * n
let days n = 24 * 60 * 60 * n
       
let get_tmin i = 
  match i with
      1 -> minutes 10
    | 2 -> minutes 20
    | 3 -> minutes 40
    | 4 -> hours 1
    | 5 -> hours 2
    | 6 -> hours 4
    | 7 -> hours 8
    | i -> 
        if i <= 0 then (
          FATAL "Non-positive index: %d" i;
          -1
        ) else
          let j = i - 8 in days (power 2 j)

let get_tmax i =
  match i with
      1 -> minutes 30
    | 2 -> hours 1
    | 3 -> hours 2
    | 4 -> hours 4
    | 5 -> hours 8
    | i -> 
        if i <= 0 then (
          FATAL "Non-positive index: %d" i;
          -1
        ) else
          let j = i - 6 in days (power 2 j)

let mAX_TIME_DIFF = days 30
    
let compute_weight time voc = 
  let i = 
    match voc.voc_index with
        Some i -> i
      | None   -> 1
  in 
  let t = 
    (* It's safe to use normal ints for the time diff *)
    match voc.voc_time with
        Some t -> min mAX_TIME_DIFF (Int32.to_int (Int32.sub time t))
      | None   -> mAX_TIME_DIFF
  in
  let _ = DTRACE "compute_weight: timediff = %d, index = %d" t i in
  let rec compute_alpha j =
    if t <= get_tmax j then j
    else compute_alpha (j + 1)
  in
  let tmin = get_tmin i in
  let tmax = get_tmax i in
  let w =
    if t < tmin then 0.0
    else if t <= tmax then 1.0 /. (float_of_int i)
    else 
      let alpha = compute_alpha(i) in
        float_of_int (square (alpha - i + 1)) /.(float_of_int i)
  in 
    DTRACE "compute_weight: result = %f" w;
    w
            
               
let select session = 
  let l = 
    Database.with_read session.Session.ses_db_file
      (fun db ->
         let time = Time.time () in
         let _ = DTRACE "Computing weight of every vocable" in
           Database.map (fun key s -> 
                           let voc = Vocable.deserialize s in
                             (voc, compute_weight time voc)) db
      )
  in
    DTRACE "Done computing weight, now choosing vocable";
    let voc = WeightedChoice.choose l Vocable.get_term in
      DTRACE "Choosing vocable finished";
      voc


(************************ Tests *******************************)

let test_compute_weight () = 
  let voc = { voc_term = ["x"]; voc_meaning = ["y"]; voc_examples = [];
              voc_termPronunciation = None; voc_meaningPronunciation = None;
              voc_filename = "f"; voc_lineno = 0;
              voc_index = Some 3; 
              voc_time = Some (Int32.of_int 0);
              voc_timesAsked = 0} in
  let t1 = minutes 20 in
  let t2 = hours 1 in
  let t3 = hours 7 in
    assert (Utils.float_eq 0.0 (compute_weight (Int32.of_int t1) voc));
    assert (Utils.float_eq (1.0 /. 3.0) 
              (compute_weight (Int32.of_int t2) voc));
    assert (Utils.float_eq 3.0 (compute_weight (Int32.of_int t3) voc))

let test_power () = 
  assert (power 2 0 == 1);
  assert (power 3 3 == 27);
  assert (power 5 2 == 25);
  assert (try ignore (power 10 ~-2); false with Invalid_argument _ -> true)

let test_get_tminmax () = 
  assert (get_tmin 8 == days 1);
  assert (get_tmin 11 == days 8);
  assert (get_tmax 6 == days 1);
  assert (get_tmax 10 == days 16)
    
let test () = 
  test_power ();
  print_endline "test_power passed";
  test_get_tminmax ();
  print_endline "test_get_tminmax passed";
  test_compute_weight ();
  print_endline "test_compute_weight passed"
